# Contributing

Here is some wisdom to help you build and test this project as a developer and potential contributor.

The project template is generated with [cmake-init](https://github.com/friendlyanon/cmake-init).

If you plan to contribute, please read the [CONTRIBUTING](CONTRIBUTING.md) guide.

## Install

Only two commands are necessary to install this library on your system.

```sh
cmake -S . -B build
cmake --install build
```

## Developer mode

Build system targets that are only useful for developers of this project are hidden if the `emio_DEVELOPER_MODE` option
is disabled. Enabling this option makes tests and other developer targets and options available. Not enabling this
option means that you are a consumer of this project and thus you have no need for these targets and options.

Developer mode is always set to on in CI workflows.

### Presets

This project makes use of [presets][1] to simplify the process of configuring the project. As a developer, you are
recommended to always have the [latest CMake version][2] installed to make use of the latest Quality-of-Life additions.

You have a few options to pass `emio_DEVELOPER_MODE` to the configure command, but this project prefers to use presets.

As a developer, you should create a `CMakeUserPresets.json` file at the root of the project:

```json
{
  "version": 2,
  "cmakeMinimumRequired": {
    "major": 3,
    "minor": 14,
    "patch": 0
  },
  "configurePresets": [
    {
      "name": "dev",
      "binaryDir": "${sourceDir}/build/dev",
      "inherits": ["dev-mode", "ci-<os>"],
      "cacheVariables": {
        "CMAKE_BUILD_TYPE": "Debug"
      }
    }
  ],
  "buildPresets": [
    {
      "name": "dev",
      "configurePreset": "dev",
      "configuration": "Debug"
    }
  ],
  "testPresets": [
    {
      "name": "dev",
      "configurePreset": "dev",
      "configuration": "Debug",
      "output": {
        "outputOnFailure": true
      }
    }
  ]
}
```

You should replace `<os>` in your newly created presets file with the name of the operating system you have, which may
be `win64` or `unix`. You can see what these correspond to in the [`CMakePresets.json`](CMakePresets.json) file.

`CMakeUserPresets.json` is also the perfect place in which you can put all sorts of things that you would otherwise want
to pass to the configure command in the terminal.

### Configure, build and test

If you followed the above instructions, then you can configure, build and test the project respectively with the
following commands from the project root on any operating system with any build system:

```sh
cmake --preset=dev
cmake --build --preset=dev
ctest --preset=dev
```

Please note that both the build and test command accepts a `-j` flag to specify the number of jobs to use, which should
ideally be specified to the number of threads your CPU has. You may also want to add that to your preset using the
`jobs` property, see the [presets documentation][1] for more details.

[1]: https://cmake.org/cmake/help/latest/manual/cmake-presets.7.html

[2]: https://cmake.org/download/
